# Example using PIO to drive a set of WS2812 LEDs.
import array, rp2
from machine import Pin
from time import sleep

# Configure the number of WS2812 LEDs.
NUM_LEDS = 4
PIN_NUM = 22 

@rp2.asm_pio(sideset_init=rp2.PIO.OUT_LOW, out_shiftdir=rp2.PIO.SHIFT_LEFT, autopull=True, pull_thresh=24)
def ws2812():
    T1 = 2
    T2 = 5
    T3 = 3
    wrap_target()
    label("bitloop")
    out(x, 1)               .side(0)    [T3 - 1]
    jmp(not_x, "do_zero")   .side(1)    [T1 - 1]
    jmp("bitloop")          .side(1)    [T2 - 1]
    label("do_zero")
    nop()                   .side(0)    [T2 - 1]
    wrap()
pass
        
class RGBLed(object): #NeoPixel
    
    def __init__(self,pin=PIN_NUM,num=NUM_LEDS,brightness=0.8):
        self.pin=pin
        self.num=num
        self.brightness = brightness
        
        # Create the StateMachine with the ws2812 program, outputting on pin
        self.sm = rp2.StateMachine(0, ws2812, freq=8_000_000, sideset_base=Pin(PIN_NUM))

        # Start the StateMachine, it will wait for data on its FIFO.
        self.sm.active(1)

        # Display a pattern on the LEDs via an array of LED RGB values.
        self.ar = array.array("I", [0 for _ in range(self.num)])
        
        self.BLACK = (0, 0, 0)
        self.RED = (255, 0, 0)
        self.YELLOW = (255, 150, 0)
        self.GREEN = (0, 255, 0)
        self.CYAN = (0, 255, 255)
        self.BLUE = (0, 0, 255)
        self.PURPLE = (180, 0, 255)
        self.WHITE = (255, 255, 255)
        self.COLORS = (self.BLACK, self.RED, self.YELLOW, self.GREEN, self.CYAN, self.BLUE, self.PURPLE, self.WHITE)
    pass
        
    def pixels_show(self):
        dimmer_ar = array.array("I", [0 for _ in range(self.num)])
        
        for i,c in enumerate(self.ar):
            r = int(((c >> 8) & 0xFF) * self.brightness)
            g = int(((c >> 16) & 0xFF) * self.brightness)
            b = int((c & 0xFF) * self.brightness)
            dimmer_ar[i] = (g<<16) + (r<<8) + b
        pass
    
        self.sm.put(dimmer_ar, 8)
    pass

    def set_color( self, color ):
        self.pixels_set(0, color)
        self.pixels_set(1, color)
        self.pixels_set(2, color)
        self.pixels_set(3, color)
        
        self.pixels_show()
    pass

    def pixels_set(self, i, color):
        self.ar[i] = (color[1]<<16) + (color[0]<<8) + color[2]
    pass

    def pixels_fill(self, color):
        for i in range(len(self.ar)) :
            self.pixels_set(i, color)
        pass
    pass

    def color_chase(self, color, wait):
        for i in range(self.num):
            self.pixels_set(i, color)
            sleep(wait)
            self.pixels_show()
        sleep(0.2)
    pass
     
    def wheel(self, pos):
        # Input a value 0 to 255 to get a color value.
        # The colours are a transition r - g - b - back to r.
        if pos < 0 or pos > 255:
            return (0, 0, 0)
        if pos < 85:
            return (255 - pos * 3, pos * 3, 0)
        if pos < 170:
            pos -= 85
            return (0, 255 - pos * 3, pos * 3)
        pos -= 170
        
        return (pos * 3, 0, 255 - pos * 3)
    pass
     
     
    def rainbow_cycle(self, wait):
        for j in range(255):
            for i in range(self.num):
                rc_index = (i * 256 // self.num) + j
                self.pixels_set(i, self.wheel(rc_index & 255))
            self.pixels_show()
            sleep(wait)
        pass
    pass

pass

if __name__=='__main__':
    
    rgbLed = RGBLed()
    print("fills")
    for color in rgbLed.COLORS:       
        rgbLed.pixels_fill(color)
        rgbLed.pixels_show()
        sleep(0.5)

    print("chases")
    for color in strip.COLORS:       
        rgbLed.color_chase(color, 0.05)

    print("rainbow")
    while True :
        rgbLed.rainbow_cycle(0.05)
    pass

pass



from machine import UART, Pin
import ujson, utime

from .TRSensor import TRSensor
from .Motor import PicoGo
from .RGBLed import RGBLed
from .LCD import LCD


bat = machine.ADC(Pin(26))
temp = machine.ADC(4)

lcd = LCD()
lcd.fill(0xF232)
lcd.line(2,2,70,2,0xBB56)
lcd.line(70,2,85,17,0xBB56)
lcd.line(85,17,222,17,0xBB56)
lcd.line(222,17,237,32,0xBB56)
lcd.line(2,2,2,118,0xBB56)
lcd.line(2,118,16,132,0xBB56)
lcd.line(17,132,237,132,0xBB56)
lcd.line(237,32,237,132,0xBB56)

lcd.text("Raspberry Pi Pico",90,7,0xFF00)
lcd.text("PicoGo",10,7,0x001F)
lcd.text("Waveshare.com",70,120,0x07E0)
lcd.show()

IR = Pin(5, Pin.IN)
M = PicoGo()
uart = UART(0, 9600)     # init with given baudrate
led = Pin(25, Pin.OUT)
led.value(1)
BUZ = Pin(4, Pin.OUT)
BUZ.value(0)

Buzzer = Pin(4, Pin.OUT)
DSR = Pin(2, Pin.IN)
DSL = Pin(3, Pin.IN)
Echo = Pin(15, Pin.IN)
Trig = Pin(14, Pin.OUT)
Trig.value(0)
Echo.value(0)

strip = RGBLed()
strip.pixels_set(0, strip.BLACK)
strip.pixels_set(1, strip.BLACK)
strip.pixels_set(2, strip.BLACK)
strip.pixels_set(3, strip.BLACK)
strip.pixels_show()

LOW_SPEED    =  30
MEDIUM_SPEED =  50
HIGH_SPEED   =  80

speed = 20
t = 0
j = 0
n = 0

def dist():
    Trig.value(1)
    utime.sleep_ms(10)
    Trig.value(0)
    while(Echo.value() == 0):
        pass
    ts=utime.ticks_us()
    while(Echo.value() == 1):
        pass
    te=utime.ticks_us()
    distance=((te-ts)*0.034)/2
    
    return distance
pass

while True:    
    D = dist()
    
    DR_status = DSR.value()
    DL_status = DSL.value()
    
    if((utime.ticks_ms() - t) > 3000):
        t=utime.ticks_ms()
        reading = temp.read_u16() * 3.3 / (65535)
        temperature = 27 - (reading - 0.706)/0.001721
        v = bat.read_u16()*3.3/65535 * 2
        p = (v - 3) * 100 / 1.2
        if(p < 0):p=0
        if(p > 100):p=100

        lcd.fill_rect(145,50,50,40,0xF232)
        lcd.text("temperature :  {:5.2f} C".format(temperature),30,50,0xFFFF)
        lcd.text("Voltage     :  {:5.2f} V".format(v),30,65,0xFFFF)
        lcd.text("percent     :   {:3.1f} %".format(p),30,80,0xFFFF)
        lcd.show()
    print(D)
    if(D<5):
        M.stop()
    elif((DL_status == 0) and (DR_status == 1)):
        M.left(20)
    elif((DL_status == 1) and (DR_status == 0)):
        M.right(20)
    elif(((D>5) and( D<7)) or ((DL_status == 0) and (DR_status == 0))):
        M.forward(30)
    else:
       M.stop() 
        
    utime.sleep_ms(20)
    

    for i in range(strip.num):
        strip.pixels_set(i, strip.wheel(((i * 256 // strip.num) + j) & 255))
    strip.pixels_show()
    
    j += 1
    
    if(j > 256): 
        j = 0
    pass

pass


__name__ = "picogo"
__package__ = "picogo"
__version__ = '0.0.7'
__author__ = "SkySLAM Co., Ltd."

from .Robot import (
    Robot , 
)

from .MainFileCopy import main_file_copy

print( "picogo __init__.py executed." ) 


from machine import Pin
from time import sleep, sleep_us, ticks_us

class UltraSonic :
    
    def __init__( self ) :
        self.echo = Pin(15, Pin.IN)
        self.trig = Pin(14, Pin.OUT)
        
        self.trig.value(0)
        self.echo.value(0)
    pass

    def obstacle_distance( self ) :
        echo = self.echo
        trig = self.trig
        
        trig.value(1)
        sleep_us( 10 )
        trig.value(0)
        
        while echo.value() == 0 :
            pass
        
        then = ticks_us()
        
        while echo.value() == 1 :
            pass
        
        now = ticks_us()
        
        distance= (now - then)*0.017
        
        return distance
    pass

pass



from machine import Pin
from time import sleep

from .Motor import Motor
from .UltraSonic import UltraSonic

if __name__ == '__main__' :

    motor = Motor()
    ultraSonic = UltraSonic()
    
    dsr = Pin(2, Pin.IN)
    dsl = Pin(3, Pin.IN)
    
    duration = 0.02
    speed = 20
    max_dist = 18
    
    left_block = False
    right_block = False 
    obstacle_cnt = 0
        
    while True:
        dist = ultraSonic.obstacle_distance()
        
        left_block = ( dsr.value() == 0 )
        right_block = ( dsl.value()== 0 )
        
        if dist < max_dist or left_block or right_block == 0 :
            obstacle_cnt += 1
        pass
    
        if obstacle_cnt == 1 :
            motor.stop()
            sleep( duration)
        pass
    
        if left_block and right_block : # 양쪽에 장애물이 있을 때, 좌회전
            motor.left( speed )
            sleep( 5*duration )
        elif left_block :   # 좌측 장애물시, 우회전
            motor.right( speed )
            sleep( 5*duration )
        elif right_block :  # 우측 장애물시, 좌회전
            motor.left( speed )
            sleep( 5*duration )
        if dist < max_dist : # 전방 장애물 있을 경우, 좌회전 
            motor.left( speed )
            sleep( 5*duration )        
        else :              # 장매물이 없으면, 전진
            obstacle_cnt = 0 
            motor.forward( speed )
            sleep( duration )
        pass
    pass

pass



from machine import Pin

class IRSensor :
    
    def __init__( self ) :
        self.dsr = Pin(2, Pin.IN)
        self.dsl = Pin(3, Pin.IN)
    pass

    def read_blocks( self ) :
        return [ self.dsl.value() == 0, self.dsr.value() == 0 ]   # left_block, right_block
    pass

pass


from machine import Pin
import time

from .TRSensor import TRSensor
from .Motor import PicoGo
from .RGBLed import RGBLed


M = PicoGo()
Buzzer = Pin(4, Pin.OUT)
DSR = Pin(2, Pin.IN)
DSL = Pin(3, Pin.IN)

strip = RGBLed()
strip.pixels_set(0, strip.RED)
strip.pixels_set(1, strip.GREEN)
strip.pixels_set(2, strip.BLUE)
strip.pixels_set(3, strip.YELLOW)
strip.pixels_show()  
time.sleep(2)

TRS=TRSensor()
for i in range(100):
    if(i<25 or i>= 75):
        M.setMotor(30,-30)
    else:
        M.setMotor(-30,30)
    TRS.calibrate()
print("\ncalibrate done\r\n")
print(TRS.calibratedMin)
print(TRS.calibratedMax)
print("\ncalibrate done\r\n")
maximum = 100
integral = 0
last_proportional = 0
j=0

while True:
    position,Sensors = TRS.readLine()
    DR_status = DSR.value()
    DL_status = DSL.value()
    
    if((Sensors[0] + Sensors[1] + Sensors[2]+ Sensors[3]+ Sensors[4]) > 4000):
        Buzzer.value(0)
        M.setMotor(0,0)
    elif((DL_status == 0) or (DR_status == 0)):
        Buzzer.value(1)
        M.setMotor(0,0)
    else:
        Buzzer.value(0)
        # The "proportional" term should be 0 when we are on the line.
        proportional = position - 2000

        # Compute the derivative (change) and integral (sum) of the position.
        derivative = proportional - last_proportional
        #integral += proportional

        # Remember the last position.
        last_proportional = proportional
        
        '''
        // Compute the difference between the two motor power settings,
        // m1 - m2.  If this is a positive number the robot will turn
        // to the right.  If it is a negative number, the robot will
        // turn to the left, and the magnitude of the number determines
        // the sharpness of the turn.  You can adjust the constants by which
        // the proportional, integral, and derivative terms are multiplied to
        // improve performance.
        '''
        power_difference = proportional/30  + derivative*2;  

        if (power_difference > maximum):
            power_difference = maximum
        if (power_difference < - maximum):
            power_difference = - maximum

        if (power_difference < 0):
            M.setMotor(maximum + power_difference, maximum)
        else:
            M.setMotor(maximum, maximum - power_difference)

    for i in range(strip.num):
        strip.pixels_set(i, strip.wheel(((i * 256 // strip.num) + j) & 255))
    strip.pixels_show()
    
    j += 1
    if(j > 256): 
        j = 0
    pass

pass



import machine

sdaPin = machine.Pin(7)
sclPin = machine.Pin(8)
i2c = machine.I2C(0, sda=sdaPin, scl=sclPin, freq=400000)

devices = i2c.scan()

if len(devices) == 0:
    print('No i2c device found!')
else:
    print('i2c device found', len(devices))

for device in devices:
    print("At address: ", hex(device))


from time import sleep 

from .Motor import Motor

if __name__== '__main__' :
    motor = Motor()
    
    duration = 1
    
    motor.forward(30)
    sleep(duration)
    
    motor.backward(30)
    sleep(duration)
    
    motor.left(30)
    sleep(duration)
    
    motor.right(30)
    sleep(duration)
    
    motor.stop()
    sleep(duration)
pass


if __name__== '__main__' :
    from machine import Pin, Timer
    led = Pin(25, Pin.OUT)
    timer = Timer()

    def blink(timer):
        led.toggle()

    timer.init(freq=2, mode=Timer.PERIODIC, callback=blink)
    
pass



from time import sleep

from .Motor import PicoGo
from .UltraSonic import UltraSonic

if __name__ == '__main__' :
    print( "Hello" )

    robot = PicoGo()
    ultraSonic = UltraSonic()
    
    duration = 0.02
    speed = 20
    max_dist = 18
    
    obstacle_cnt = 0 
    while True :
        dist = ultraSonic.obstacle_distance()
        
        print( f"distance = {dist:6.2f} cm" )
        
        if dist < max_dist :
            ostacle_cnt += 1
            
            if obstacle_cnt == 1 : 
                robot.stop()
                sleep( duration )
            
            robot.right( 2*speed )
        else:
            ostacle_cnt = 0 
            robot.forward( speed )
        pass
            
        sleep( duration )
    pass

    print( "Goodbye!" )
pass


from machine import Pin, ADC
from time import sleep

class Battery :
    def __init__( self ) : 
        self.batt = ADC(Pin(26))
        self.temp = ADC(4)
    pass

    def read( self ) :
        batt = self.batt
        temp = self.temp
        
        reading = temp.read_u16()*3.3/65535        
        temperature = 27 - (reading - 0.706)/0.001721
        
        voltage = batt.read_u16()*3.3/65535*2
        percent = (voltage - 3)*100/1.2
        
        percent = max( 0, min( 100, percent ) )
        
        return temperature, voltage, percent         
    pass

pass